@@ -1,9 +1,9 @@
 from .ConsensusClustering import ConsensusClustering
 from .art_of_tsne import tsne
-from .balanced_pca import balanced_pca, significant_pc_test, log_scale, get_pc_centers
+from .balanced_pca import balanced_pca, significant_pc_test, log_scale, get_pc_centers, ReproduciblePCA
 from .scrublet import MethylScrublet
 from .pvclust import Dendrogram
 from .dmg import PairwiseDMG, one_vs_rest_dmg
-from .feature_enrichment import cluster_enriched_features
+from .feature_selection.feature_enrichment import cluster_enriched_features
 from .mcad import filter_regions, lsi, binarize_matrix, remove_black_list_region
-from .integration import calculate_direct_confusion
\ No newline at end of file
+from .integration import calculate_direct_confusion
