@@ -6,10 +6,48 @@ from pybedtools import BedTool
 import dask
 import re
 import warnings
+import pathlib
+import glob
 from .utilities import highly_variable_methylation_feature, calculate_posterior_mc_frac
 from ..plot.qc_plots import cutoff_vs_cell_remain, plot_dispersion
 
 
+def _determine_engine(mcds_paths):
+    def _single_path(path):
+        if pathlib.Path(f'{path}/.zgroup').exists():
+            e = 'zarr'
+        else:
+            e = None  # default for None is netcdf4
+        return e
+
+    def _multi_paths(paths):
+        engines = []
+        for path in paths:
+            e = _single_path(path)
+            engines.append(e)
+        engine = list(set(engines))
+        if len(engine) > 1:
+            raise ValueError(f'Can not open a mixture of netcdf4 and zarr files in "{mcds_paths}"')
+        else:
+            engine = engine[0]
+        return engine
+
+    if isinstance(mcds_paths, str):
+        if '*' in mcds_paths:
+            engine = _multi_paths(list(glob.glob(mcds_paths)))
+        else:
+            # single mcds path
+            engine = _single_path(mcds_paths)
+    else:
+        engine = _multi_paths(mcds_paths)
+
+    if engine is None:
+        print(f'Open MCDS with netcdf4 engine.')
+    else:
+        print(f'Open MCDS with {engine} engine.')
+    return engine
+
+
 class MCDS(xr.Dataset):
     __slots__ = ()
 
@@ -19,7 +57,7 @@ class MCDS(xr.Dataset):
         return
 
     @classmethod
-    def open(cls, mcds_paths, obs_dim='cell', use_obs=None):
+    def open(cls, mcds_paths, obs_dim='cell', use_obs=None, split_large_chunks=True):
         """
         Take one or multiple MCDS file paths and create single MCDS concatenated on obs_dim
 
@@ -31,21 +69,26 @@ class MCDS(xr.Dataset):
             Dimension name of observations, default is 'cell'
         use_obs
             Subset the MCDS by a list of observation IDs.
+        split_large_chunks
+            Whether split large chunks in dask config array.slicing.split_large_chunks
         Returns
         -------
         MCDS
         """
+        engine = _determine_engine(mcds_paths)
         if isinstance(mcds_paths, str) and '*' not in mcds_paths:
-            ds = xr.open_dataset(mcds_paths)
+            ds = xr.open_dataset(mcds_paths, engine=engine)
         else:
-            with dask.config.set(**{'array.slicing.split_large_chunks': False}):
+            with dask.config.set(**{'array.slicing.split_large_chunks': split_large_chunks}):
                 ds = xr.open_mfdataset(mcds_paths,
                                        parallel=False,
                                        combine='nested',
-                                       concat_dim=obs_dim)
+                                       concat_dim=obs_dim,
+                                       engine=engine)
         if use_obs is not None:
-            use_obs = ds.get_index(obs_dim).intersection(use_obs)
-            ds = ds.sel({obs_dim: use_obs})
+            with dask.config.set(**{'array.slicing.split_large_chunks': split_large_chunks}):
+                use_obs_bool = ds.get_index(obs_dim).isin(use_obs)
+                ds = ds.sel({obs_dim: use_obs_bool})
         return cls(ds).squeeze()
 
     def add_mc_frac(self,
@@ -252,7 +295,8 @@ class MCDS(xr.Dataset):
         # remove bad features
         judge = (feature_mc_frac_mean > 0) & (feature_std > 0) & (feature_cov_mean > 0)
         if n_top_feature >= judge.size:
-            raise ValueError('n_top_feature must be smaller than total number of features')
+            n_top_feature = judge.size
+            print('n_top_feature is than total number of features, will use all features')
         feature_mc_frac_mean = feature_mc_frac_mean[judge]
         feature_var = feature_std[judge] ** 2  # to be consistent with previous bin-based method, use var here
         feature_cov_mean = feature_cov_mean[judge]
@@ -401,7 +445,8 @@ class MCDS(xr.Dataset):
             self.coords[f'{var_dim}_{mc_type}_{name}'] = column
         return hvf_df
 
-    def get_adata(self, mc_type, var_dim, da_suffix='frac', obs_dim='cell', select_hvf=True):
+    def get_adata(self, mc_type, var_dim, da_suffix='frac', obs_dim='cell', select_hvf=True,
+                  split_large_chunks=True):
         """
         Get anndata from MCDS mC rate matrix
         Parameters
@@ -416,46 +461,51 @@ class MCDS(xr.Dataset):
             Name of observation
         select_hvf
             Select HVF or not, if True, will use mcds.coords['{var_dim}_{mc_type}_feature_select'] to select HVFs
+        split_large_chunks
+            Whether split large chunks in dask config array.slicing.split_large_chunks
 
         Returns
         -------
         anndata.Anndata
         """
-        if select_hvf:
-            try:
-                use_features = self.get_index(var_dim)[self.coords[f'{var_dim}_{mc_type}_feature_select']]
-                use_data = self[f'{var_dim}_da_{da_suffix}'].sel({'mc_type': mc_type, var_dim: use_features}).squeeze()
-            except KeyError:
-                print('feature_select==True, but no highly variable feature results found, use all features instead.')
+        with dask.config.set(**{'array.slicing.split_large_chunks': split_large_chunks}):
+            if select_hvf:
+                try:
+                    use_features = self.get_index(var_dim)[self.coords[f'{var_dim}_{mc_type}_feature_select']]
+                    use_data = self[f'{var_dim}_da_{da_suffix}'].sel(
+                        {'mc_type': mc_type, var_dim: use_features}).squeeze()
+                except KeyError:
+                    print('feature_select==True, but no highly variable feature results found, '
+                          'use all features instead.')
+                    use_data = self[f'{var_dim}_da_{da_suffix}'].sel({'mc_type': mc_type}).squeeze()
+            else:
                 use_data = self[f'{var_dim}_da_{da_suffix}'].sel({'mc_type': mc_type}).squeeze()
-        else:
-            use_data = self[f'{var_dim}_da_{da_suffix}'].sel({'mc_type': mc_type}).squeeze()
 
-        obs_df = pd.DataFrame([], index=use_data.get_index(obs_dim).astype(str))
-        var_df = pd.DataFrame([], index=use_data.get_index(var_dim).astype(str))
-        coord_prefix = re.compile(f'({obs_dim}|{var_dim})_')
-        for k, v in use_data.coords.items():
-            if k in [obs_dim, var_dim]:
-                continue
-            try:
-                # v.dims should be size 1
-                if v.dims[0] == obs_dim:
-                    series = v.to_pandas()
-                    # adata.obs_name is str type
-                    series.index = series.index.astype(str)
-                    obs_df[coord_prefix.sub('', k)] = series
-                elif v.dims[0] == var_dim:
-                    series = v.to_pandas()
-                    # adata.var_name is str type
-                    series.index = series.index.astype(str)
-                    var_df[coord_prefix.sub('', k)] = series
-                else:
+            obs_df = pd.DataFrame([], index=use_data.get_index(obs_dim).astype(str))
+            var_df = pd.DataFrame([], index=use_data.get_index(var_dim).astype(str))
+            coord_prefix = re.compile(f'({obs_dim}|{var_dim})_')
+            for k, v in use_data.coords.items():
+                if k in [obs_dim, var_dim]:
+                    continue
+                try:
+                    # v.dims should be size 1
+                    if v.dims[0] == obs_dim:
+                        series = v.to_pandas()
+                        # adata.obs_name is str type
+                        series.index = series.index.astype(str)
+                        obs_df[coord_prefix.sub('', k)] = series
+                    elif v.dims[0] == var_dim:
+                        series = v.to_pandas()
+                        # adata.var_name is str type
+                        series.index = series.index.astype(str)
+                        var_df[coord_prefix.sub('', k)] = series
+                    else:
+                        pass
+                except IndexError:
+                    # v.dims is 0, just ignore
                     pass
-            except IndexError:
-                # v.dims is 0, just ignore
-                pass
 
-        adata = anndata.AnnData(X=use_data.transpose(obs_dim, var_dim).values,
-                                obs=obs_df,
-                                var=var_df)
+            adata = anndata.AnnData(X=use_data.transpose(obs_dim, var_dim).values,
+                                    obs=obs_df,
+                                    var=var_df)
         return adata
